﻿//UTF-8
using System;

namespace PictureInDatabase
{
    public partial class frmMain : System.Windows.Forms.Form
    {

        private const string SQL_SELECT = "SELECT [imagefield] FROM [tablename] WHERE ([keyfield] = @keyfield);";
        private const string SQL_INSERT = "INSERT INTO [tablename]([imagefield], [keyfield]) VALUES(@imagefield ,@keyfield);";
        private const string SQL_UPDATE = "UPDATE [tablename] SET [imagefield] = @imagefield WHERE ([keyfield] = @keyfield);";
        private const string SQL_DELETE = "DELETE FROM [tablename] WHERE ([keyfield] = @keyfield);";
        private const string SQL_EXISTS = "SELECT COUNT(*) FROM [tablename] WHERE ([keyfield] = @keyfield);";

        private const string CONNECTIONSTRING = "Provider=Microsoft.Jet.OLEDB.4.0;Persist Security Info=False;Data Source=\"|DataDirectory|DataBase.mdb\"";

        //////////////////////////////////////////////////////////////////////
        public frmMain()
        {
            this.InitializeComponent();
        }

#region  " Events "

        //////////////////////////////////////////////////////////////////////
        private void btnLoad_Click(object sender, System.EventArgs e)
        {
            this.f_Picture.Image = this.LoadImage((int)(this.f_ID.Value));
        }

        //////////////////////////////////////////////////////////////////////
        private void btnSave_Click(object sender, System.EventArgs e)
        {
            if (
                (this.OFD.ShowDialog(this) != System.Windows.Forms.DialogResult.OK) ||
                (String.IsNullOrEmpty(this.OFD.FileName))) return;
            //------------------------------------------------------------
            int ikeyfield = (int)(this.f_ID.Value);
            this.SaveImage(ikeyfield, this.OFD.FileName);
            this.f_Picture.Image = this.LoadImage(ikeyfield);
       }

        //////////////////////////////////////////////////////////////////////
        private void btnDelete_Click(object sender, System.EventArgs e)
        {
            int ikeyfield = (int)(this.f_ID.Value);
            this.DeleteImage(ikeyfield);
            this.f_Picture.Image = this.LoadImage(ikeyfield);
       }

#endregion

#region  " Engine "

        //////////////////////////////////////////////////////////////////////
        private System.Drawing.Image LoadImage(int keyfield)
        {
            try
            {
                using (System.Data.OleDb.OleDbConnection dbCon = new System.Data.OleDb.OleDbConnection(CONNECTIONSTRING)){
                using (System.Data.OleDb.OleDbCommand dbCom = new System.Data.OleDb.OleDbCommand(SQL_SELECT, dbCon)){

                    dbCom.Parameters.AddWithValue("@keyfield", keyfield);

                    dbCon.Open();
                    byte[] buffer = (byte[])dbCom.ExecuteScalar();
                    dbCon.Close();

                    if ((buffer != null) && (buffer.Length > 0))
                    {
                        using (System.IO.MemoryStream strm = new System.IO.MemoryStream(buffer))
                        {
                            return System.Drawing.Image.FromStream(strm);
                        }
                    }
                }}
            }
            catch {
                //return null;
            }
            //------------------------------------------------------------
            return null;
        }

        //////////////////////////////////////////////////////////////////////
        private bool SaveImage(int keyfield, string imagefilepath)
        {
            try
            {
                using (System.Drawing.Image bmp = System.Drawing.Image.FromFile(imagefilepath))
                {
                    if (bmp == null) return false;
                    return this.SaveImage(keyfield, bmp);
                }
            }
            catch {
                return false;
            }
        }

        //////////////////////////////////////////////////////////////////////
        private bool SaveImage(int keyfield, System.Drawing.Image imagefield)
        {
            bool bIsExist;
            //------------------------------------------------------------
            try
            {
                using (System.Data.OleDb.OleDbConnection dbCon = new System.Data.OleDb.OleDbConnection(CONNECTIONSTRING)){
                using (System.Data.OleDb.OleDbCommand dbCom = new System.Data.OleDb.OleDbCommand(SQL_EXISTS, dbCon)){

                    dbCom.Parameters.AddWithValue("@keyfield", keyfield);

                    dbCon.Open();

                    try
                    {
                        bIsExist = (int)dbCom.ExecuteScalar() > 0;
                    }
                    catch {
                        bIsExist = false;
                    }
                    //------------------------------------------------------------
                    dbCom.Parameters.Clear();

                    dbCom.Parameters.AddRange(
                       new System.Data.OleDb.OleDbParameter[] {
                       new System.Data.OleDb.OleDbParameter("@imagefield", System.Data.OleDb.OleDbType.VarBinary, 0, System.Data.ParameterDirection.Input, true, ((byte)0), ((byte)0), null, System.Data.DataRowVersion.Current, null),
                       new System.Data.OleDb.OleDbParameter("@keyfield", System.Data.OleDb.OleDbType.Integer, 0, System.Data.ParameterDirection.Input, false, ((byte)0), ((byte)0), null, System.Data.DataRowVersion.Current, keyfield)});

                    if (imagefield == null)
                        dbCom.Parameters[0].Value = System.DBNull.Value;
                    else
                    {
                        using (System.IO.MemoryStream strm = new System.IO.MemoryStream())
                        {
                            imagefield.Save(strm, System.Drawing.Imaging.ImageFormat.Jpeg);
                            dbCom.Parameters[0].Value = strm.ToArray();
                        }
                    }
                    //------------------------------------------------------------
                    dbCom.CommandText = bIsExist ? SQL_UPDATE: SQL_INSERT;
                    //------------------------------------------------------------
                    return dbCom.ExecuteNonQuery() > 0;
                }}
            }
            catch {
                return false;
            }
        }

        //////////////////////////////////////////////////////////////////////
        private bool DeleteImage(int keyfield)
        {
            try
            {
                using (System.Data.OleDb.OleDbConnection dbCon = new System.Data.OleDb.OleDbConnection(CONNECTIONSTRING)){
                using (System.Data.OleDb.OleDbCommand dbCom = new System.Data.OleDb.OleDbCommand(SQL_DELETE, dbCon)){

                    dbCom.Parameters.AddWithValue("@keyfield", keyfield);

                    dbCon.Open();
                    return dbCom.ExecuteNonQuery() > 0;

                }}
            }
            catch {
                return false;
            }
        }

#endregion


        //////////////////////////////////////////////////////////////////////
        [STAThread()] public static void Main()
        {
            System.Windows.Forms.Application.EnableVisualStyles();
            using (frmMain f = new frmMain())
            {
                System.Windows.Forms.Application.Run(f);
            }
       }



    }
}
